from botocore.config import Config


class S3Client(object):
    def __init__(self, boto_session, boto_config=None, endpoint_url=None):
        """

        @param boto_session: pre-existing boto session
        @param boto_config: optional boto config for the s3 client
        @param endpoint_url: optional endpoint for the s3 client used for localstack
        """

        default_config = Config(signature_version="s3v4")
        if boto_config is None:
            boto_config = default_config
        else:
            # default config values will take precedence if non-default.
            # s3 version defaults to v2.
            # This will not work in regions created past Jan. 2014:
            # http://docs.aws.amazon.com/AmazonS3/latest/API/sig-v4-authenticating-requests.html
            boto_config = default_config.merge(boto_config)

        self.s3_client = boto_session.client(
            "s3", config=boto_config, endpoint_url=endpoint_url
        )
        self.s3_resource = boto_session.resource(
            "s3", config=boto_config, endpoint_url=endpoint_url
        )

    def download_file(self, file_name, bucket_name, key_name):
        """
        Download an S3 object to a file

        @param file_name:
        @param bucket_name: target s3 bucket
        @param key_name: The name of the key to download from
        """
        self.s3_client.download_file(
            Bucket=bucket_name, Key=key_name, Filename=file_name
        )

    def download_fileobj(self, file_obj, bucket_name, key_name):
        """
        Download an object from S3 to a file-like object

        @param file_obj: A file-like object to download into
        @param bucket_name:  The name of the bucket to download from
        @param key_name: The name of the key to download from
        """
        self.s3_client.download_fileobj(
            Bucket=bucket_name, Key=key_name, Fileobj=file_obj
        )
        file_obj.seek(0)
